<?php

/**
 * @file
 * Contains functions for the admin interface of pay-per-node module.
 * 
 * This file contains function for building the admin interface: overview
 * of all user's balances, bulk operations, allowed types and more. 
 */

/*
 *  paypernode administration interface
 *
 */  
function paypernode_admin_overview() {
  $output = '';

  $query = "SELECT u.uid, u.name, SUM(p.allowed) AS allowed, SUM(p.created) AS created FROM {users} u " .
           "INNER JOIN {paypernode_user} p ON u.uid = p.uid GROUP BY u.uid, u.name ORDER BY u.uid";
  $result = pager_query($query, 30, 0, "SELECT COUNT(DISTINCT(u.uid)) FROM {users} u INNER JOIN {paypernode_user} p ON u.uid = p.uid");

  global $pager_total_items;
 
  // Present results
  if ($pager_total_items[0]>0) {
    $header = array(t('user'), t('total'), t('created'), t('left'), t('administer'));
    while ($user = db_fetch_object($result)) {
      $rows[] = array(
        theme('username', $user),
        $user->allowed,
        $user->created,
        $user->allowed - $user->created,
        l(t('edit'), "user/$user->uid/paypernode")
      );
    }
    $output .= theme('table', $header, $rows);
    $output .= theme('pager', array(), 30);
  } 
  else {
    $output .= t("No Pay per node data for users");
  }
  return $output;
}

/**
 *  Admin can setup here which types an user can create through pay-per-node system
 *
 */   
function paypernode_admin_types() {
    return drupal_get_form('paypernode_admin_types_form');
}

function paypernode_admin_types_form($form_state) {
  $form = array(); 
  $options=paypernode_get_types(TRUE);
  if (count($options)) {
    $form['type']=array(
        '#title' => t('Select one or more content types'),
        '#tree'  => TRUE,
        '#type' => 'fieldset',
    );
    foreach ($options as $type_system_name => $object) {
         $form['type'][$type_system_name] = array(
          '#type' => 'checkbox',
          '#title' => check_plain($object->name),
          '#required' => TRUE,
          '#default_value' => variable_get('paypernode_type_' . $type_system_name, FALSE),
        );
    }
    $form['submit'] = array('#type' => 'submit', '#value' => t('Submit')); 
  } 
  else {
    $form['submit'] = array('#value' => t('No content types available.')); 
  }
  return $form;
}

function paypernode_admin_types_form_submit($form, $form_state) {
    $values=isset($form_state['values']['type']) ? $form_state['values']['type'] : array();
    foreach (paypernode_get_types(TRUE) as $value => $name) {
        if (isset($values[$value]) && $values[$value]==1) {
          variable_set('paypernode_type_' . $value, TRUE);
        } 
        else {
          variable_set('paypernode_type_' . $value, FALSE);
        } 
    }
    drupal_set_message(t('Allowed content types updated.'));
}

/**
 * Bulk account administration
 * Allows to add a number of nodes for a given role
 */
function paypernode_admin_bulk() {
  return drupal_get_form('paypernode_admin_bulk_form');
}

function paypernode_admin_bulk_form($form_state) {
  $ppn_types=paypernode_get_types();
  $options=array();
  $form = array();
  if (count($ppn_types)) {
    foreach ($ppn_types as $ppn_system_name => $ppn_object) {
        $options[$ppn_system_name]=$ppn_object->name;
    }
    $form['bulk']=array(
        '#title' => t('Bulk operations'),
        '#type' => 'fieldset',
    );
    $form['bulk']['role'] = array(
          '#type' => 'radios',
          '#title' => t('Role'),
          '#default_value' => '',
          '#options' => user_roles(1),
          '#required' => TRUE,
          '#description' => t("User's role to perform the operation."),
    );
    $form['bulk']['node_type'] = array(
          '#type' => 'radios',
          '#title' => t('Node type'),
          '#default_value' => '',
          '#required' => TRUE,
          '#options' => $options,
          '#description' => t("Node type to perform the operation."),
    );
    $form['bulk']['amount'] = array(
          '#type' => 'textfield',
          '#size' => 4, 
          '#maxlength' => 4,
          '#required' => TRUE,
          '#title' => t('Number of nodes'),
          '#default_value' => 0,
          '#description' => t("Number of nodes to add. This can be negative."),
    );
    $form['submit'] = array(
        '#type' => 'submit', 
        '#value' => t('Submit')
    );
  } 
  else {
    $form['submit'] = array(
        '#value' => t('No content types available (see !link)', array('!link' => l('allowed content types', 'admin/settings/paypernode/allowed')))
    ); 
  }
  return $form;
}

function paypernode_admin_bulk_form_submit($form, &$form_state) {
  $rid = $form_state['values']['role'];
  $node_type = $form_state['values']['node_type'];
  $amount = $form_state['values']['amount'];

  // Manage exception for "authenticated user", rid = 2, that is not in users_roles table
  if ($rid == 2) {
    $sql = "SELECT u.uid, p.allowed, p.created FROM {users} u LEFT JOIN {paypernode_user} p ON u.uid = p.uid AND p.node_type = '%s' WHERE u.uid > 0";
    $params = array($node_type);
  } 
  else {
    $sql = "SELECT u.uid, p.allowed, p.created FROM {users} u INNER JOIN {users_roles} r ON u.uid = r.uid LEFT JOIN {paypernode_user} p ON u.uid = p.uid AND p.node_type = '%s' WHERE r.rid = '%d'";
    $params = array($node_type, $rid);
  }

  // Update user accounts
  $result = db_query($sql, $params);

  $user_count = 0;
  $node_count = 0;
  while ($user = db_fetch_object($result)) {
    if (is_numeric($user->allowed)) {
      $total = $user->allowed + $amount;
      // Check before update for negative balance and allowed >= created
      if ($total >= 0 && $total >= $user->created) {
        paypernode_user_update($user->uid, $node_type, $amount);
        $user_count++;
        $node_count += $amount;
      }
      elseif ($user->created < $user->allowed) {
        // Just set balance to null, allowed = created
        paypernode_user_update($user->uid, $node_type, $user->created, $user->created, 'set');
        $user_count++;
        $node_count += $user->created - $user->allowed;
      } // else nothing to do for this user
    }
    elseif ($amount > 0) {
      // User has no previous balance of that type. Do not allow negative balance
      paypernode_user_update($user->uid, $node_type, $amount, 0, 'set');
      $user_count++;
      $node_count += $amount;
    }
  }
  drupal_set_message(t('Updated %user_count user accounts. Added %node_count nodes.', array('%user_count' => $user_count, '%node_count' => $node_count)));
}

